@@ -471,7 +471,8 @@ class HardwareIOTypeForm(forms.Form):
 
 from django.forms.models import inlineformset_factory, BaseModelFormSet, BaseInlineFormSet, BaseModelForm
 from data.models.models import HardwareStats, DataTransformer, DataTransformerConstants, HardwareDataTransformer, \
-    HardwareStatsDataTransformer
+    HardwareStatsDataTransformer, DataTransformerTypes
+
 
 # DataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer, fk_name='data_transformer_parent',
 #                                                      extra=1,exclude =['updated_at','created_at'],formset=DataTransformerForm)
@@ -480,28 +481,29 @@ class DataConstantsTransformerForm(forms.ModelForm):
     id = forms.CharField(required=False,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
     type = forms.CharField(required=False,
-                                widget=forms.TextInput(attrs={'class': 'form-control'}))
+                           widget=forms.TextInput(attrs={'class': 'form-control'}))
 
     index = forms.CharField(required=False,
-                                              widget=forms.HiddenInput(attrs={'class': 'form-control'}))
+                            widget=forms.HiddenInput(attrs={'class': 'form-control'}))
     value = forms.CharField(required=False,
-                           widget=forms.TextInput(attrs={'class': 'form-control'}))
+                            widget=forms.TextInput(attrs={'class': 'form-control'}))
+
     class Meta:
         model = DataTransformerConstants
         fields = ('__all__')
 
+
 class HardwareTransformerForm(forms.ModelForm):
     id = forms.CharField(required=False,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
     index = forms.CharField(required=False,
                             widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
-
-
     class Meta:
         model = HardwareDataTransformer
         fields = ('__all__')
 
+
 class HardwareStatsTransformerForm(forms.ModelForm):
     id = forms.CharField(required=False,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
@@ -512,9 +514,13 @@ class HardwareStatsTransformerForm(forms.ModelForm):
         model = HardwareStatsDataTransformer
         fields = ('__all__')
 
-DataConstantsTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformerConstants, extra=1,form=DataConstantsTransformerForm)
-HardwareTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareDataTransformer, extra=1,form=HardwareTransformerForm)
-HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=1,form=HardwareStatsTransformerForm)
+
+DataConstantsTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformerConstants, extra=1,
+                                                              form=DataConstantsTransformerForm)
+HardwareTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareDataTransformer, extra=1,
+                                                         form=HardwareTransformerForm)
+HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=1,
+                                                              form=HardwareStatsTransformerForm)
 
 from django.utils.safestring import mark_safe
 
@@ -526,12 +532,14 @@ class DataTransform(forms.ModelForm):
                                 widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     parent = forms.CharField(required=False,
-                                              widget=forms.HiddenInput(attrs={'class': 'form-control'}))
-
+                             widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     def __init__(self, *args, **kwargs):
         print('Init Data Transform')
         super(DataTransform, self).__init__(*args, **kwargs)
+        types = list(
+            DataTransformerTypes.objects.all().values_list('type', 'type'))
+        self.fields['type'] = ChoiceField(choices=tuple(types))
         try:
             if hasattr(self.instance, 'get_children') and self.instance.get_children().count() > 0:
                 print('Adding to child form node: ' + str(self.instance.get_children().count() + 1))
@@ -544,7 +552,7 @@ class DataTransform(forms.ModelForm):
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': self.instance,}]
+                    initial=[{'parent': self.instance, }]
                 )
                 self.nested_constants_form = DataConstantsTransformerInlineFormset(
                     instance=self.instance,
@@ -566,15 +574,12 @@ class DataTransform(forms.ModelForm):
         except Exception:
             print('excpt')
 
-
-
-
     def as_p(self):
         print('wtf')
         print('instance: ' + str(self.instance.id))
         forms = '<div class="row"><div class="col-6">'
 
-        forms = super(DataTransform,self).as_p()
+        forms = super(DataTransform, self).as_p()
         forms = forms + '</div><div class="col-6">'
 
         if hasattr(self, 'nested_constants_form') and self.nested_constants_form is not None:
@@ -592,7 +597,9 @@ class DataTransform(forms.ModelForm):
 
     class Meta:
         model = DataTransformer
-        fields = ['id','accessory','parent','type']
+        fields = ['id', 'accessory', 'parent', 'type']
+
+
 # DataTransformerFormset = form_factory(DataTransformer,formset=DataTransformerForm)
 
 
@@ -602,14 +609,14 @@ class DataTransformRoot(forms.ModelForm):
     accessory = forms.CharField(required=False,
                                 widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
-
-
-
     # def __init__(self, *args, **kwargs):
     #     super(HardwareTypeForm, self).__init__(*args, **kwargs)
     def __init__(self, *args, **kwargs):
         super(DataTransformRoot, self).__init__(*args, **kwargs)
         print('Adding to root form node: ' + str(self.instance.get_children().count() + 1))
+        types = list(
+            DataTransformerTypes.objects.all().values_list('type', 'type'))
+        self.fields['type'] = ChoiceField(choices=tuple(types))
 
         dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
                                                              fk_name='parent', form=DataTransform,
@@ -619,7 +626,7 @@ class DataTransformRoot(forms.ModelForm):
             queryset=self.instance.get_children(),
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            initial=[{'parent': self.instance,}]
+            initial=[{'parent': self.instance, }]
         )
 
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
@@ -645,7 +652,7 @@ class DataTransformRoot(forms.ModelForm):
         print('instance: ' + str(self.instance.id))
         forms = '<div class="row"><div class="col-6">'
 
-        forms = forms + super(DataTransformRoot,self).as_p()
+        forms = forms + super(DataTransformRoot, self).as_p()
         forms = forms + '</div><div class="col-6">'
         if hasattr(self, 'nested_constants_form') and self.nested_constants_form is not None:
             forms = forms + '\n  -' + self.nested_constants_form.as_p()
@@ -655,11 +662,11 @@ class DataTransformRoot(forms.ModelForm):
             forms = forms + '\n  -' + self.nested_hardware_stats_form.as_p()
         if hasattr(self, 'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
             print('netdatatransform with number elements: ' + str(len(self.nested_data_transformer_form)))
-            forms = forms + '\n       -' +  ' '.join(form.as_p() for form in self.nested_data_transformer_form)
+            forms = forms + '\n       -' + ' '.join(form.as_p() for form in self.nested_data_transformer_form)
         forms = forms + '</div></div>'
         return mark_safe(forms)
 
     class Meta:
         model = DataTransformer
-        fields = ['id','accessory','type']
+        fields = ['id', 'accessory', 'type']
 # DataTransformerFormset = form_factory(DataTransformer,formset=DataTransformerForm)
